from django.utils.datastructures import MultiValueDict, MergeDict

def ExpandableWidgetFactory(base_widget, min_values, max_values):
    class _ExpandableWidget(base_widget):
        def __getattr__(self, name):
            return getattr(base_widget, name)

        def value_from_datadict(self, data, files, name):
            data_dict = getattr(self, 'input_type', None) == 'file' and files or data
            if isinstance(data_dict, (MultiValueDict, MergeDict)):
                values = data_dict.getlist(name)
            else:
                values = data_dict.get(name)
            return self._pad(values)

        def _pad(self, values):
            # pad values as necessary
            if not values:
                values = [''] * min_values
            elif len(values) < min_values:
                values += [''] * (min_values - len(values))
            return values

        def _render_generator_factory(self):
            def render_generator(name, values, attrs):
                # delete the 'id' key from attrs if it exists because an expandable field should not have id
                if isinstance(attrs, dict) and 'id' in attrs:
                    del attrs['id']

                values = self._pad(values)
                for v in values:  
                    yield super(_ExpandableWidget, self).render(name, v, attrs)
                else:
                    raise StopIteration
            return render_generator

        def render(self, name, values, attrs=None):
            if not hasattr(self, '_render_generator'):
                # values must be an iterator or a list
                self._render_generator = self._render_generator_factory()(name, values, attrs)

            try:
                return self._render_generator.next()
            except StopIteration:
                # when list is exhausted, render empty widget
                return super(_ExpandableWidget, self).render(name, '', attrs)
    return _ExpandableWidget


def ExpandableWidget(base_widget, min_values=0, max_values=None, **kwargs):
    """
    Given a widget, returns an expandable version of the widget.
    An expandable widget accepts zero or more values in GET or POST data. For example,

    <form name="fav_food_form">
        <input type="text" name="food"></input>
        <input type="text" name="food"></input>
        <input type="text" name="food"></input>
        ...
    </form>
    
    class FavFoodForm(forms.Form):
        food = ExpandableField(CharField, widget=ExpandableWidget(widgets.TextInput))
        ...

        def clean_food(self):
            list_of_food = self.cleaned_data['food']  # this is a list of strings
    """
    return ExpandableWidgetFactory(base_widget, min_values, max_values)(**kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django import forms
from django.forms.util import ValidationError
from xfield.widgets import ExpandableWidget 
import logging

def ExpandableField(base_field, min_values=0, max_values=None, **kwargs):
    class _ExpandableField(base_field):
        def __init__(self, **kwargs):
            self.min_values = min_values
            self.max_values = max_values  # WARNING: max_values is not implemented

            if not 'widget' in kwargs:
                widget = ExpandableWidget(base_field.widget, min_values, max_values)
            else:
                widget = kwargs.pop('widget')
                # check the type of the widget. we can't do isinstance() or issubclass() without some magic because
                # expandable field is an instance of a dynamic class
                widget_type = type(widget)
                if widget_type.__name__ != '_ExpandableWidget':
                    raise AttributeError("'%s' is not an instance of ExpandableWidget" % widget_type.__name__)

                if callable(widget):
                    widget = widget()

            if 'initial' in kwargs:
                initial = kwargs['initial']
                if not hasattr(initial, '__iter__'):
                    raise AttributeError("ExpandableField accepts only iterable initial data.")

            super(_ExpandableField, self).__init__(widget=widget, **kwargs)

        def clean(self, values, initial=None):
            values = values is None and [u""] or list(values)
            has_error = False
            errors = ['' for i in range(len(values))]
            for i, v in enumerate(values):
                # if no value provided for this expandable field, check if user has satisfied the number of minimum
                # values
                if v:
                    try:
                        # If ExpandableField wraps around a FileField, the field's clean() method must be passed a second
                        # argument; @see django/forms/forms.py, line 282
                        if isinstance(base_field, forms.FileField):
                            values[i] = super(_ExpandableField, self).clean(v, initial)
                        else:
                            values[i] = super(_ExpandableField, self).clean(v)
                    except ValidationError, e:
                        errors[i] = e.messages[0]  # we are only interested in the first error message
                        has_error = True
                    except Exception, e:
                        logging.error(e)
                else:
                    if i >= self.min_values:  # min_values is 1-based, i is 0-based
                        # so we got the number of required values, continue
                        values[i] = v
                    else:
                        try:
                            errors[i] = self.default_error_messages['required']
                        except KeyError:
                            errors[i] = self.default_error_messages['invalid']
                        has_error = True

            if has_error:
                raise ValidationError(errors)
            else:
                return values
    return _ExpandableField(**kwargs)



from distutils.core import setup

with open('README.rst') as readme:
    long_description = readme.read()

setup(
    name='django-xfield',
    version='0.1.0',
    url='https://github.com/phuihock/django-xfield',
    license='GPLv3',
    author='Chang Phui Hock',
    author_email='phuihock@gmail.com',
    description="A Django utility package to handle zero or more form inputs of the same name.",
    long_description=long_description,
    include_package_data=True,
    packages=['xfield'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
    ],
)
